//
// Hopfield Network
//

nslImport nslAllImports;

nslImport HopfieldIn;
nslImport HopfieldOut;

nslModel Hopfield () {
    
    private int MAXSIZE=6;
    private int MAXSIZE2=MAXSIZE*MAXSIZE;
    private NslInt0 dis();

    public NslDinDouble2 input(MAXSIZE,MAXSIZE);
    public NslDouble1 pattern(MAXSIZE2);
    public NslDouble1 net(MAXSIZE2);
    public NslDouble2 weight(MAXSIZE2,MAXSIZE2);
    public NslDoutDouble0 energy();
    public NslDoutDouble2 output(MAXSIZE,MAXSIZE);

    public HopfieldIn inModule(MAXSIZE);
    public HopfieldOut outModule(MAXSIZE,MAXSIZE2);  

    // Nsl Methods
	
    public void makeConn() {
	
	nslConnect(inModule.input, this.input);
	nslConnect(this.energy, outModule.energy);
	nslConnect(this.output, outModule.output);
	
    }

    public void initSys() {
	system.setRunEndTime(150);
	system.setTrainEndTime(1);
    }
    
    public void initModule() {
    	system.nslSetTrainDelta(1);
	system.nslSetRunDelta(1);

	weight=0;
    }
       
    public void simTrain() {
	nslPrint("Adding new pattern to the net\n");
		
	for(int i=0;i<MAXSIZE; i++) {
	    for(int j=0; j<MAXSIZE; j++) {
		pattern[i*MAXSIZE+j]=input[i][j];
	    }
	}

	for(int i=0;i<MAXSIZE2; i++) {
	    for(int j=0; j<MAXSIZE2; j++) {
		weight[i][j]=((i==j)?0.0:weight[i][j]+pattern[i]*pattern[j]/MAXSIZE2);
	    }
	}
    }
	
    public void initRun() {
    
    	for(int i=0;i<MAXSIZE; i++) {
	    for(int j=0; j<MAXSIZE; j++) {
		net[i*MAXSIZE+j]=input[i][j];
	    }
	}

	for(int i=0; i<dis.get();) {
	    int r = (int) (Math.random()*MAXSIZE2);
	    double temp; 
	    temp = -1 * net[r];
	    if  (temp!=input[r/MAXSIZE][r%MAXSIZE]) {
		net[r]=temp;
		i++;
	    }
	}
    }
	
    public void simRun() {
		
	int i = (int) (Math.random()*MAXSIZE2);
				
	double sum=0.0;
	for (int j=0 ; j<MAXSIZE2; j++) {
	    sum = sum + weight[i][j] * net[j];
	}
		
	net[i]=((sum>=0)?1:-1);
		
	for(i=0;i<MAXSIZE; i++) {
	    for(int j=0; j<MAXSIZE; j++) {
		output[i][j]=net[i*MAXSIZE+j];
	    }
	}

	sum=0;
	for(i=0;i<MAXSIZE2; i++) {
	    for(int j=0; j<MAXSIZE2; j++) {
		sum=sum+weight[i][j]*net[i]*net[j];
	    }
	}
	
	energy=-.5*sum;
    }
	
    
}
